<?php

namespace SGalinski\SgJobs\SignalSlot;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Frontend\Page\PageRepository;
use TYPO3\Languagevisibility\Service\FrontendServices;

/**
 * Signal functions regarding sg_seo sitemap generation
 */
class SitemapSignalSlot {
	public const PLUGIN_NAME = 'sgjobs_jobapplication';

	/**
	 * Before the sitemap is generated
	 *
	 * @param array $pageList
	 * @param int $sysLanguageUid
	 * @throws \TYPO3\CMS\Core\Cache\Exception\NoSuchCacheException
	 */
	public function beforeSitemapGeneration(array &$pageList, int $sysLanguageUid): void {
		// find sites where job detail plugin is added
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
			->getQueryBuilderForTable('tt_content');
		$databaseResource = $queryBuilder->select('pid', 'pages')
			->from('tt_content')
			->where(
				$queryBuilder->expr()->andX(
					$queryBuilder->expr()->eq('CType', $queryBuilder->createNamedParameter('list')),
					$queryBuilder->expr()->eq(
						'list_type',
						$queryBuilder->createNamedParameter(self::PLUGIN_NAME)
					),
					$queryBuilder->expr()->orX(
						$queryBuilder->expr()->eq('sys_language_uid', -1),
						$queryBuilder->expr()->eq(
							'sys_language_uid',
							$queryBuilder->createNamedParameter($sysLanguageUid, \PDO::PARAM_INT)
						)
					)
				)
			)
			->execute();

		$rows = $databaseResource->fetchAll();
		foreach ($rows as $row) {
			$jobs = $this->getJobsByPid($row['pages'], $sysLanguageUid);

			foreach ($jobs as $job) {
				$url = $GLOBALS['TSFE']->cObj->typoLink_URL(
					[
						'parameter' => $row['pid'],
						'linkAccessRestrictedPages' => 1,
						'forceAbsoluteUrl' => 1,
						'additionalParams' => GeneralUtility::implodeArrayForUrl(
							NULL, ['tx_sgjobs_jobapplication[jobId]' => $job['uid']]
						),
					]
				);

				$pageList[] = [
					'url' => htmlspecialchars($url),
					'title' => ''
				];
			}
		}
	}

	/**
	 * Get all jobs stored on specific page ids
	 *
	 * @param string $pageList
	 * @param int $sysLanguageUid
	 * @return array|null
	 * @throws \TYPO3\CMS\Core\Cache\Exception\NoSuchCacheException
	 * @throws \Exception
	 */
	private function getJobsByPid(string $pageList, int $sysLanguageUid): ?array {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
			->getQueryBuilderForTable('tx_sgjobs_domain_model_job');
		$databaseResource = $queryBuilder->select('*')
			->from('tx_sgjobs_domain_model_job')
			->where(
				$queryBuilder->expr()->andX(
					$queryBuilder->expr()->in('pid', $queryBuilder->createNamedParameter($pageList)),
					$queryBuilder->expr()->in('sys_language_uid', '-1,0')
				)
			)
			->execute();

		$visibleRows = [];
		$rows = $databaseResource->fetchAll();
		$pageRepository = GeneralUtility::makeInstance(PageRepository::class);
		$isLanguageVisibilityLoaded = ExtensionManagementUtility::isLoaded('languagevisibility');
		foreach ($rows as $row) {
			$table = 'tx_sgjobs_domain_model_job';

			// get languagevisibility uid that is available (check for the correct uid to fall back to)
			if ($isLanguageVisibilityLoaded) {
				$element = FrontendServices::getElement($row['uid'], $table);
				$languageId = FrontendServices::getOverlayLanguageIdForElement($element, $sysLanguageUid);
				if ($languageId === FALSE) {
					continue;
				}
			} else {
				$languageId = $sysLanguageUid;
			}

			if ($languageId > 0) {
				$row = $pageRepository->getRecordOverlay($table, $row, $languageId);
			}

			$visibleRows[] = $row;
		}

		return $visibleRows;
	}
}
